=== modified file 'NEWS'
--- NEWS	2009-04-16 21:49:48 +0000
+++ NEWS	2009-04-07 18:35:19 +0000
@@ -5,7 +5,7 @@
 ---------------
 
     - Loggerhead now serves bzr branches over HTTP and exposes the URL
-      to branch them. Addresses bug #240577. (Jonathan Lange)
+      to branch them. (Jonathan Lange)
 
     - Leading blank lines in commit messages no longer result in an
       empty summary. (Colin Watson)
@@ -46,9 +46,6 @@
     - serve-branches now has an option, --use-cdn, to load YUI from
       Yahoo!'s CDN. (Matt Nordhoff)
 
-    - Fix bug #361238 (serve-branches was calling config.get_option()
-      incorrectly and failing). (Matt Nordhoff)
-
 1.10 [22Dec2008]
 ---------------
 

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-04-30 11:15:58 +0000
+++ loggerhead/apps/branch.py	2009-04-07 18:29:10 +0000
@@ -38,31 +38,27 @@
         self.branch_link = branch_link  # Currently only used in Launchpad
         self.log = logging.getLogger('loggerhead.%s' % friendly_name)
         if graph_cache is None:
-            graph_cache = bzrlib.lru_cache.LRUCache(10)
+            graph_cache = bzrlib.lru_cache.LRUCache()
         self.graph_cache = graph_cache
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
 
     def get_history(self):
-        file_cache = None
-        revinfo_disk_cache = None
+        _history = History(self.branch, self.graph_cache)
         cache_path = self._config.get('cachepath', None)
         if cache_path is not None:
             # Only import the cache if we're going to use it.
             # This makes sqlite optional
             try:
-                from loggerhead.changecache import (
-                    FileChangeCache, RevInfoDiskCache)
+                from loggerhead.changecache import FileChangeCache
             except ImportError:
                 self.log.debug("Couldn't load python-sqlite,"
                                " continuing without using a cache")
             else:
-                file_cache = FileChangeCache(cache_path)
-                revinfo_disk_cache = RevInfoDiskCache(cache_path)
-        return History(
-            self.branch, self.graph_cache, file_cache=file_cache,
-            revinfo_disk_cache=revinfo_disk_cache, cache_key=self.friendly_name)
+                _history.use_file_cache(
+                    FileChangeCache(_history, cache_path))
+        return _history
 
     def url(self, *args, **kw):
         if isinstance(args[0], list):
@@ -119,11 +115,7 @@
             self._static_url_base = self._url_base
         self._environ = environ
         if self.served_url is _DEFAULT:
-            public_branch = self.branch_url()
-            if public_branch is not None:
-                self.served_url = public_branch
-            else:
-                self.served_url = self.url([])
+            self.served_url = self.url([])
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(

=== modified file 'loggerhead/apps/filesystem.py'
--- loggerhead/apps/filesystem.py	2009-05-04 18:53:21 +0000
+++ loggerhead/apps/filesystem.py	2009-04-07 18:29:10 +0000
@@ -1,6 +1,8 @@
 """Serve branches at urls that mimic the file system layout."""
 
-from bzrlib import branch, errors, lru_cache, urlutils
+import os
+
+from bzrlib import branch, errors, lru_cache
 
 from paste.request import path_info_pop
 from paste import httpexceptions
@@ -14,11 +16,11 @@
 
 class BranchesFromFileSystemServer(object):
 
-    def __init__(self, transport, root, name=None):
-        self.transport = transport
+    def __init__(self, path, root, name=None):
+        self.path = path
         self.root = root
         self.name = name
-        self._config = root._config
+        self._config = LoggerheadConfig()
 
     def app_for_branch(self, branch):
         if not self.name:
@@ -45,22 +47,22 @@
             else:
                 name = '/'
             return DirectoryUI(environ['loggerhead.static.url'],
-                               self.transport,
+                               self.path,
                                name)
         else:
-            new_transport = self.transport.clone(segment)
+            new_path = os.path.join(self.path, segment)
             if self.name:
-                new_name = urlutils.join(self.name, segment)
+                new_name = os.path.join(self.name, segment)
             else:
                 new_name = '/' + segment
-            return BranchesFromFileSystemServer(new_transport, self.root, new_name)
+            return BranchesFromFileSystemServer(new_path, self.root, new_name)
 
     def __call__(self, environ, start_response):
+        if not os.path.isdir(self.path):
+            raise httpexceptions.HTTPNotFound()
         try:
-            b = branch.Branch.open_from_transport(self.transport)
+            b = branch.Branch.open(self.path)
         except errors.NotBranchError:
-            if not self.transport.listable() or not self.transport.has('.'):
-                raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
             return self.app_for_branch(b)(environ, start_response)
@@ -68,10 +70,9 @@
 
 class BranchesFromFileSystemRoot(object):
 
-    def __init__(self, transport, config):
-        self.graph_cache = lru_cache.LRUCache(10)
-        self.transport = transport
-        self._config = config
+    def __init__(self, folder):
+        self.graph_cache = lru_cache.LRUCache()
+        self.folder = folder
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
@@ -82,24 +83,23 @@
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
         elif '/.bzr/' in environ['PATH_INFO']:
-            app = urlparser.make_static(None, self.transport)
+            app = urlparser.make_static(None, self.folder)
             return app(environ, start_response)
         else:
             return BranchesFromFileSystemServer(
-                self.transport, self)(environ, start_response)
+                self.folder, self)(environ, start_response)
 
 
 class UserBranchesFromFileSystemRoot(object):
 
-    def __init__(self, transport, config):
-        self.graph_cache = lru_cache.LRUCache(10)
-        self.transport = transport
-        self._config = config
-        self.trunk_dir = config.get_option('trunk_dir')
+    def __init__(self, folder, trunk_dir):
+        self.graph_cache = lru_cache.LRUCache()
+        self.folder = folder
+        self.trunk_dir = trunk_dir
 
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
-        path_info = environ['PATH_INFO']
+        path_info= environ['PATH_INFO']
         if path_info.startswith('/static/'):
             segment = path_info_pop(environ)
             assert segment == 'static'
@@ -110,10 +110,10 @@
             # segments starting with ~ are user branches
             if path_info.startswith('/~'):
                 segment = path_info_pop(environ)
-                new_transport = self.transport.clone(segment[1:])
+                new_path = os.path.join(self.folder, segment[1:])
                 return BranchesFromFileSystemServer(
-                    new_transport, self, segment)(environ, start_response)
+                    new_path, self, segment)(environ, start_response)
             else:
-                new_transport = self.transport.clone(self.trunk_dir)
+                new_path = os.path.join(self.folder, self.trunk_dir)
                 return BranchesFromFileSystemServer(
-                    new_transport, self)(environ, start_response)
+                    new_path, self)(environ, start_response)

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2009-05-02 14:01:05 +0000
+++ loggerhead/changecache.py	2009-03-19 21:01:11 +0000
@@ -27,10 +27,8 @@
 """
 
 import cPickle
-import marshal
 import os
 import tempfile
-import zlib
 
 try:
     from sqlite3 import dbapi2
@@ -40,26 +38,16 @@
 # We take an optimistic approach to concurrency here: we might do work twice
 # in the case of races, but not crash or corrupt data.
 
-def safe_init_db(filename, init_sql):
-    # To avoid races around creating the database, we create the db in
-    # a temporary file and rename it into the ultimate location.
-    fd, temp_path = tempfile.mkstemp(dir=os.path.dirname(filename))
-    os.close(fd)
-    con = dbapi2.connect(temp_path)
-    cur = con.cursor()
-    cur.execute(init_sql)
-    con.commit()
-    con.close()
-    os.rename(temp_path, filename)
-
 class FakeShelf(object):
 
     def __init__(self, filename):
         create_table = not os.path.exists(filename)
         if create_table:
-            safe_init_db(
-                filename, "create table RevisionData "
-                "(revid binary primary key, data binary)")
+            # To avoid races around creating the database, we create the db in
+            # a temporary file and rename it into the ultimate location.
+            fd, path = tempfile.mkstemp(dir=os.path.dirname(filename))
+            self._create_table(path)
+            os.rename(path, filename)
         self.connection = dbapi2.connect(filename)
         self.cursor = self.connection.cursor()
 
@@ -101,7 +89,8 @@
 
 class FileChangeCache(object):
 
-    def __init__(self, cache_path):
+    def __init__(self, history, cache_path):
+        self.history = history
 
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
@@ -115,44 +104,3 @@
             changes = self.history.get_file_changes_uncached(entry)
             cache.add(entry.revid, changes)
         return changes
-
-
-class RevInfoDiskCache(object):
-    """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
-
-    def __init__(self, cache_path):
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-        filename = os.path.join(cache_path, 'revinfo.sql')
-        create_table = not os.path.exists(filename)
-        if create_table:
-            safe_init_db(
-                filename, "create table Data "
-                "(key binary primary key, revid binary, data binary)")
-        self.connection = dbapi2.connect(filename)
-        self.cursor = self.connection.cursor()
-
-    def get(self, key, revid):
-        self.cursor.execute(
-            "select revid, data from data where key = ?", (dbapi2.Binary(key),))
-        row = self.cursor.fetchone()
-        if row is None:
-            return None
-        elif str(row[0]) != revid:
-            return None
-        else:
-            return marshal.loads(zlib.decompress(row[1]))
-
-    def set(self, key, revid, data):
-        try:
-            self.cursor.execute(
-                'delete from data where key = ?', (dbapi2.Binary(key), ))
-            blob = zlib.compress(marshal.dumps(data))
-            self.cursor.execute(
-                "insert into data (key, revid, data) values (?, ?, ?)",
-                map(dbapi2.Binary, [key, revid, blob]))
-            self.connection.commit()
-        except dbapi2.IntegrityError:
-            # If another thread or process attempted to set the same key, we
-            # don't care too much -- it's only a cache after all!
-            pass

=== modified file 'loggerhead/config.py'
--- loggerhead/config.py	2009-04-16 22:28:17 +0000
+++ loggerhead/config.py	2009-04-07 18:29:10 +0000
@@ -4,14 +4,6 @@
 import tempfile
 
 
-_temporary_sql_dir = None
-
-def _get_temporary_sql_dir():
-    global _temporary_sql_dir
-    if _temporary_sql_dir is None:
-        _temporary_sql_dir = tempfile.mkdtemp(prefix='loggerhead-cache-')
-    return _temporary_sql_dir
-
 def command_line_parser():
     parser = OptionParser("%prog [options] <path>")
     parser.set_defaults(
@@ -19,7 +11,6 @@
         show_version=False,
         log_folder=None,
         use_cdn=False,
-        sql_dir=None,
         )
     parser.add_option("--user-dirs", action="store_true", dest="user_dirs",
                       help="Serve user directories as ~user.")
@@ -47,22 +38,17 @@
                       help="Print the software version and exit")
     parser.add_option('--use-cdn', action='store_true',
                       help="Serve YUI from Yahoo!'s CDN")
-    parser.add_option('--cache-dir', dest='sql_dir',
-                      help="The directory to place the SQL cache in")
     return parser
 
 
-class LoggerheadConfig(object):
+class LoggerheadConfig:
     '''A configuration object.'''
 
     def __init__(self):
         self._parser = command_line_parser()
         self._options, self._args = self._parser.parse_args(sys.argv[1:])
 
-        sql_dir = self.get_option('sql_dir')
-        if sql_dir is None:
-            sql_dir = _get_temporary_sql_dir()
-        self.SQL_DIR = sql_dir
+        self.SQL_DIR = tempfile.mkdtemp(prefix='loggerhead-cache-')
 
     def get_option(self, option):
         '''Get an option from the options dict.'''

=== modified file 'loggerhead/controllers/directory_ui.py'
--- loggerhead/controllers/directory_ui.py	2009-05-04 18:53:21 +0000
+++ loggerhead/controllers/directory_ui.py	2008-12-05 18:52:44 +0000
@@ -18,7 +18,7 @@
 
 import datetime
 import logging
-import stat
+import os
 
 from bzrlib import branch
 
@@ -48,7 +48,7 @@
 
     template_path = 'loggerhead.templates.directory'
 
-    def __init__(self, static_url_base, transport, name):
+    def __init__(self, static_url_base, path, name):
 
         class _branch(object):
             context_url = 1
@@ -58,23 +58,24 @@
                 return self._static_url_base + path
         self._branch = _branch
         self._history_callable = lambda:None
+        self._path = path
         self._name = name
         self._static_url_base = static_url_base
-        self.transport = transport
         self.log = logging.getLogger('')
 
     def get_values(self, path, kwargs, response):
-        listing = self.transport.list_dir('.')
+        listing = [d for d in os.listdir(self._path)
+                   if not d.startswith('.')
+                   and os.path.isdir(os.path.join(self._path, d))]
         listing.sort(key=lambda x: x.lower())
         dirs = []
         parity = 0
         for d in listing:
+            p = os.path.join(self._path, d)
             try:
-                b = branch.Branch.open_from_transport(self.transport.clone(d))
+                b = branch.Branch.open(p)
             except:
                 b = None
-                if not stat.S_ISDIR(self.transport.stat(d).st_mode):
-                    continue
             dirs.append(DirEntry(d, parity, b))
             parity = 1 - parity
         # Create breadcrumb trail

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-04-24 10:30:15 +0000
+++ loggerhead/history.py	2009-03-19 20:04:24 +0000
@@ -31,7 +31,6 @@
 import bisect
 import datetime
 import logging
-import marshal
 import re
 import textwrap
 import threading
@@ -48,7 +47,6 @@
 import bzrlib.delta
 import bzrlib.diff
 import bzrlib.errors
-import bzrlib.lru_cache
 import bzrlib.progress
 import bzrlib.revision
 import bzrlib.textfile
@@ -179,43 +177,6 @@
                 file_id=file_id))
 
 
-class RevInfoMemoryCache(object):
-    """A store that validates values against the revids they were stored with.
-
-    We use a unique key for each branch.
-
-    The reason for not just using the revid as the key is so that when a new
-    value is provided for a branch, we replace the old value used for the
-    branch.
-
-    There is another implementation of the same interface in
-    loggerhead.changecache.RevInfoDiskCache.
-    """
-
-    def __init__(self, cache):
-        self._cache = cache
-
-    def get(self, key, revid):
-        """Return the data associated with `key`, subject to a revid check.
-
-        If a value was stored under `key`, with the same revid, return it.
-        Otherwise return None.
-        """
-        cached = self._cache.get(key)
-        if cached is None:
-            return None
-        stored_revid, data = cached
-        if revid == stored_revid:
-            return data
-        else:
-            return None
-
-    def set(self, key, revid, data):
-        """Store `data` under `key`, to be checked against `revid` on get().
-        """
-        self._cache[key] = (revid, data)
-
-
 class History (object):
     """Decorate a branch to provide information for rendering.
 
@@ -224,76 +185,13 @@
     around it, serve the request, throw the History object away, unlock the
     branch and throw it away.
 
-    :ivar _file_change_cache: An object that caches information about the
-        files that changed between two revisions.
-    :ivar _rev_info: A list of information about revisions.  This is by far
-        the most cryptic data structure in loggerhead.  At the top level, it
-        is a list of 3-tuples [(merge-info, where-merged, parents)].
-        `merge-info` is (seq, revid, merge_depth, revno_str, end_of_merge) --
-        like a merged sorted list, but the revno is stringified.
-        `where-merged` is a tuple of revisions that have this revision as a
-        non-lefthand parent.  Finally, `parents` is just the usual list of
-        parents of this revision.
-    :ivar _rev_indices: A dictionary mapping each revision id to the index of
-        the information about it in _rev_info.
-    :ivar _full_history: A list of all revision ids in the ancestry of the
-        branch, in merge-sorted order.  This is a bit silly, and shouldn't
-        really be stored on the instance...
-    :ivar _revno_revid: A dictionary mapping stringified revnos to revision
-        ids.
+    :ivar _file_change_cache: xx
     """
 
-    def _load_whole_history_data(self, caches, cache_key):
-        """Set the attributes relating to the whole history of the branch.
-
-        :param caches: a list of caches with interfaces like
-            `RevInfoMemoryCache` and be ordered from fastest to slowest.
-        :param cache_key: the key to use with the caches.
-        """
-        self._rev_indices = None
-        self._rev_info = None
-
-        missed_caches = []
-        def update_missed_caches():
-            for cache in missed_caches:
-                cache.set(cache_key, self.last_revid, self._rev_info)
-        for cache in caches:
-            data = cache.get(cache_key, self.last_revid)
-            if data is not None:
-                self._rev_info = data
-                update_missed_caches()
-                break
-            else:
-                missed_caches.append(cache)
-        else:
-            whole_history_data = compute_whole_history_data(self._branch)
-            self._rev_info, self._rev_indices = whole_history_data
-            update_missed_caches()
-
-        if self._rev_indices is not None:
-            self._full_history = []
-            self._revno_revid = {}
-            for ((_, revid, _, revno_str, _), _, _) in self._rev_info:
-                self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
-        else:
-            self._full_history = []
-            self._revno_revid = {}
-            self._rev_indices = {}
-            for ((seq, revid, _, revno_str, _), _, _) in self._rev_info:
-                self._rev_indices[revid] = seq
-                self._revno_revid[revno_str] = revid
-                self._full_history.append(revid)
-
-    def __init__(self, branch, whole_history_data_cache, file_cache=None,
-                 revinfo_disk_cache=None, cache_key=None):
+    def __init__(self, branch, whole_history_data_cache):
         assert branch.is_locked(), (
             "Can only construct a History object with a read-locked branch.")
-        if file_cache is not None:
-            self._file_change_cache = file_cache
-            file_cache.history = self
-        else:
-            self._file_change_cache = None
+        self._file_change_cache = None
         self._branch = branch
         self._inventory_cache = {}
         self._branch_nick = self._branch.get_config().get_nickname()
@@ -301,10 +199,17 @@
 
         self.last_revid = branch.last_revision()
 
-        caches = [RevInfoMemoryCache(whole_history_data_cache)]
-        if revinfo_disk_cache:
-            caches.append(revinfo_disk_cache)
-        self._load_whole_history_data(caches, cache_key)
+        whole_history_data = whole_history_data_cache.get(self.last_revid)
+        if whole_history_data is None:
+            whole_history_data = compute_whole_history_data(branch)
+            whole_history_data_cache[self.last_revid] = whole_history_data
+
+        (self._revision_graph, self._full_history, self._revision_info,
+         self._revno_revid, self._merge_sort, self._where_merged,
+         ) = whole_history_data
+
+    def use_file_cache(self, cache):
+        self._file_change_cache = cache
 
     @property
     def has_revisions(self):
@@ -314,12 +219,12 @@
         return self._branch.get_config()
 
     def get_revno(self, revid):
-        if revid not in self._rev_indices:
+        if revid not in self._revision_info:
             # ghost parent?
             return 'unknown'
-        seq = self._rev_indices[revid]
-        revno = self._rev_info[seq][0][3]
-        return revno
+        (seq, revid, merge_depth,
+         revno_str, end_of_merge) = self._revision_info[revid]
+        return revno_str
 
     def get_revids_from(self, revid_list, start_revid):
         """
@@ -333,11 +238,10 @@
 
         def introduced_revisions(revid):
             r = set([revid])
-            seq = self._rev_indices[revid]
-            md = self._rev_info[seq][0][2]
+            seq, revid, md, revno, end_of_merge = self._revision_info[revid]
             i = seq + 1
-            while i < len(self._rev_info) and self._rev_info[i][0][2] > md:
-                r.add(self._rev_info[i][0][1])
+            while i < len(self._merge_sort) and self._merge_sort[i][2] > md:
+                r.add(self._merge_sort[i][1])
                 i += 1
             return r
         while 1:
@@ -345,7 +249,7 @@
                 return
             if introduced_revisions(revid) & revid_set:
                 yield revid
-            parents = self._rev_info[self._rev_indices[revid]][2]
+            parents = self._revision_graph[revid]
             if len(parents) == 0:
                 return
             revid = parents[0]
@@ -562,10 +466,10 @@
 
         merge_point = []
         while True:
-            children = self._rev_info[self._rev_indices[revid]][1]
+            children = self._where_merged.get(revid, [])
             nexts = []
             for child in children:
-                child_parents = self._rev_info[self._rev_indices[child]][2]
+                child_parents = self._revision_graph[child]
                 if child_parents[0] == revid:
                     nexts.append(child)
                 else:

=== modified file 'loggerhead/middleware/profile.py'
--- loggerhead/middleware/profile.py	2009-04-01 22:50:25 +0000
+++ loggerhead/middleware/profile.py	2009-03-30 22:05:16 +0000
@@ -35,3 +35,75 @@
             self.lock.release()
 
 
+class MemoryProfileMiddleware(object):
+    '''Paste middleware for profiling memory with heapy.'''
+
+    def __init__(self, app, limit=40):
+        self.app = app
+        self.lock = threading.Lock()
+
+        self.type2count = {}
+        self.type2all = {}
+        self.limit = limit
+
+    def update(self):
+        try:
+            obs = sys.getobjects(0)
+        except AttributeError:
+            logging.error(
+                'Python does not have debug symbols compiled.  Memory will '
+                'not be profiled.')
+            return
+        type2count = {}
+        type2all = {}
+        for o in obs:
+            all = sys.getrefcount(o)
+
+            if type(o) is str and o == '<dummy key>':
+                # avoid dictionary madness
+                continue
+            t = type(o)
+            if t in type2count:
+                type2count[t] += 1
+                type2all[t] += all
+            else:
+                type2count[t] = 1
+                type2all[t] = all
+
+        ct = [(type2count[t] - self.type2count.get(t, 0),
+               type2all[t] - self.type2all.get(t, 0),
+               t)
+              for t in type2count.iterkeys()]
+        ct.sort()
+        ct.reverse()
+        printed = False
+
+        logger = logging.getLogger('loggerhead-memprofile')
+        logger.debug('*' * 20)
+        logger.debug("Loggerhead Memory Profiling")
+        i = 0
+        for delta1, delta2, t in ct:
+            if delta1 or delta2:
+                if not printed:
+                    logger.debug("%-55s %8s %8s" % ('', 'insts', 'refs'))
+                    printed = True
+
+                logger.debug("%-55s %8d %8d" % (t, delta1, delta2))
+
+                i += 1
+                if i >= self.limit:
+                    break
+
+        self.type2count = type2count
+        self.type2all = type2all
+
+    def __call__(self, environ, start_response):
+        self.lock.acquire()
+        try:
+            # We don't want to be working with the static images
+            # TODO: There needs to be a better way to do this.
+            self.update()
+            return self.app(environ, start_response)
+        finally:
+            self.lock.release()
+

=== modified file 'loggerhead/search.py'
--- loggerhead/search.py	2009-04-17 01:50:05 +0000
+++ loggerhead/search.py	2008-11-05 00:50:06 +0000
@@ -17,17 +17,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-_mod_index = None
-def import_search():
-    global errors, _mod_index, FileTextHit, RevisionHit
-    if _mod_index is not None:
-        return
-    try:
-        from bzrlib.plugins.search import errors
-        from bzrlib.plugins.search import index as _mod_index
-        from bzrlib.plugins.search.index import FileTextHit, RevisionHit
-    except ImportError:
-        _mod_index = None
+import sets
+try:
+    from bzrlib.plugins.search import errors
+    from bzrlib.plugins.search import index as _mod_index
+    from bzrlib.plugins.search.index import FileTextHit, RevisionHit
+except ImportError:
+    _mod_index = None
 
 
 def search_revisions(branch, query_list, suggest=False):
@@ -40,7 +36,6 @@
     param suggest: Optional flag to request suggestions instead of results
     return: A list for results, either revision ids or terms
     """
-    import_search()
     if _mod_index is None:
         return None # None indicates could-not-search
     try:
@@ -64,6 +59,6 @@
                     revid_list.append(result.text_key[1])
                 elif isinstance(result, RevisionHit):
                     revid_list.append(result.revision_key[0])
-            return list(set(revid_list))
+            return list(sets.Set(revid_list))
     finally:
         index._branch.unlock()

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2009-04-02 20:32:12 +0000
+++ loggerhead/tests/test_corners.py	2009-03-16 18:54:39 +0000
@@ -34,16 +34,6 @@
         res = app.get('/revision/'+newrevid)
         res.mustcontain('executable')
 
-    def test_revision_escapes_commit_message(self):
-        """XXX."""
-        self.createBranch()
-
-        msg = '<b>hi</b>'
-        self.addFileAndCommit('myfilename', msg)
-        app = self.setUpLoggerhead()
-        res = app.get('/revision/1')
-        self.assertFalse(msg in res.body)
-
     def test_empty_commit_message(self):
         """Check that an empty commit message does not break the rendering."""
         self.createBranch()

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2009-04-02 20:32:12 +0000
+++ loggerhead/util.py	2009-03-19 20:31:49 +0000
@@ -250,7 +250,7 @@
         except UnicodeDecodeError:
             s = s.decode('iso-8859-15')
 
-    s = cgi.escape(s).expandtabs().replace(' ', NONBREAKING_SPACE)
+    s = s.expandtabs().replace(' ', NONBREAKING_SPACE)
 
     return HSC.clean(s).replace('\n', '<br/>')
 

=== modified file 'loggerhead/wholehistory.py'
--- loggerhead/wholehistory.py	2009-04-24 10:30:15 +0000
+++ loggerhead/wholehistory.py	2008-12-05 20:09:14 +0000
@@ -37,10 +37,6 @@
 
 
 def compute_whole_history_data(branch):
-    """Compute _rev_info and _rev_indices for a branch.
-
-    See History.__doc__ for what these data structures mean.
-    """
     z = time.time()
 
     last_revid = branch.last_revision()
@@ -54,32 +50,30 @@
 
     _revision_graph = _strip_NULL_ghosts(parent_map)
     _full_history = []
-
-    _rev_info = []
-    _rev_indices = {}
-
+    _revision_info = {}
+    _revno_revid = {}
     if is_null(last_revid):
         _merge_sort = []
     else:
         _merge_sort = merge_sort(
             _revision_graph, last_revid, generate_revno=True)
 
-    for info in _merge_sort:
-        seq, revid, merge_depth, revno, end_of_merge = info
+    for (seq, revid, merge_depth, revno, end_of_merge) in _merge_sort:
         _full_history.append(revid)
         revno_str = '.'.join(str(n) for n in revno)
-        parents = _revision_graph[revid]
-        _rev_indices[revid] = len(_rev_info)
-        _rev_info.append([(seq, revid, merge_depth, revno_str, end_of_merge), (), parents])
+        _revno_revid[revno_str] = revid
+        _revision_info[revid] = (
+            seq, revid, merge_depth, revno_str, end_of_merge)
+
+    _where_merged = {}
 
     for revid in _revision_graph.keys():
-        if _rev_info[_rev_indices[revid]][0][2] == 0:
+        if _revision_info[revid][2] == 0:
             continue
         for parent in _revision_graph[revid]:
-            c = _rev_info[_rev_indices[parent]]
-            if revid not in c[1]:
-                c[1] = c[1] + (revid,)
+            _where_merged.setdefault(parent, set()).add(revid)
 
     log.info('built revision graph cache: %r secs' % (time.time() - z))
 
-    return (_rev_info, _rev_indices)
+    return (_revision_graph, _full_history, _revision_info,
+            _revno_revid, _merge_sort, _where_merged)

=== modified file 'serve-branches'
--- serve-branches	2009-05-04 18:53:21 +0000
+++ serve-branches	2009-04-01 15:56:24 +0000
@@ -20,7 +20,6 @@
 import sys
 
 from bzrlib.plugin import load_plugins
-from bzrlib.transport import get_transport
 
 from paste import httpserver
 from paste.httpexceptions import HTTPExceptionHandler, HTTPInternalServerError
@@ -49,9 +48,9 @@
     else:
         path = '.'
 
-    load_plugins()
-
-    transport = get_transport(path)
+    if not os.path.isdir(path):
+        print "%s is not a directory" % path
+        sys.exit(1)
 
     if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
         print "--trunk-dir is only valid with --user-dirs"
@@ -64,12 +63,13 @@
             return Reloader.restart_with_reloader()
 
     if config.get_option('user_dirs'):
-        if not config.get_option('trunk_dir'):
+        if not config.get_option['trunk_dir']:
             print "You didn't specify a directory for the trunk directories."
             sys.exit(1)
-        app = UserBranchesFromFileSystemRoot(transport, config)
+        app = UserBranchesFromFileSystemRoot(
+            path, config.get_option('trunk_dir'))
     else:
-        app = BranchesFromFileSystemRoot(transport, config)
+        app = BranchesFromFileSystemRoot(path)
 
     # setup_logging()
     logging.basicConfig()
@@ -77,7 +77,7 @@
     logger = getattr(app, 'log', logging.getLogger('loggerhead'))
     if config.get_option('log_folder'):
         logfile_path = os.path.join(
-            config.get_option('log_folder'), 'serve-branches.log')
+            config.get_option['log_folder'], 'serve-branches.log')
     else:
         logfile_path = 'serve-branches.log'
     logfile = logging.FileHandler(logfile_path, 'a')
@@ -87,6 +87,11 @@
     logfile.setLevel(logging.DEBUG)
     logger.addHandler(logfile)
 
+    if config.get_option('memory_profile'):
+        memprofile = logging.getLogger('loggerhead-memprofile')
+        memprofile.setLevel(logging.DEBUG)
+        memprofile.addHandler(logging.FileHandler('loggerhead-memprofile'))
+
     # setup_logging() #end
 
     app = TransLogger(app, logger=logger)
@@ -94,8 +99,8 @@
         from loggerhead.middleware.profile import LSProfMiddleware
         app = LSProfMiddleware(app)
     if config.get_option('memory_profile'):
-        from dozer import Dozer
-        app = Dozer(app)
+        from loggerhead.middleware.profile import MemoryProfileMiddleware
+        app = MemoryProfileMiddleware(app)
 
     if not config.get_option('user_prefix'):
         prefix = '/'
@@ -135,6 +140,8 @@
     else:
         host = config.get_option('user_host')
 
+    load_plugins()
+
     httpserver.serve(app, host=host, port=port)
 
 

